/* This file contains some misc. stuff for use with my Synchronet Javascript
 * scripts on my BBS, Digital Distortion.
 *
 * Author: Eric Oulashin (AKA Nightfox)
 * BBS: Digital Distortion
 * BBS address: digdist.bbsindex.com
 *
 * Date       User              Description
 * 2010-01-04 Eric Oulashin     Created
 * 2010-01-13 Eric Oulashin     Added postMsgToSubBoard().
 */

load("sbbsdefs.js");

// Determine the location of this script (its startup directory).
// The code for figuring this out is a trick that was created by Deuce,
// suggested by Rob Swindell.  I've shortened the code a little.
var gStartupPath = '.';
try { throw dig.dist(dist); } catch(e) { gStartupPath = e.fileName; }
gStartupPath = backslash(gStartupPath.replace(/[\/\\][^\/\\]*$/,''));

/////////////////////////////////////////////////////////////////////////////
// Functions

// Returns a random character from a selection of SQL-safe characters.
function randomChar()
{
   if (typeof(randomChar.randStr) != "string")
      randomChar.randStr = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()~-_=+[]{}|/?><,.;:*-+";
   return (randomChar.randStr.charAt(random(randomChar.randStr.length)));
}

// Returns an MD5 code for a string (using my MD5 PHP script).
//
// Paramters:
//  pStr: The string to MD5-encode
//
// Return value: The MD5-encoded version of pStr
function md5(pStr)
{
   if (typeof(pStr) != "string")
      return "";

   var md5Str = "";
   // TODO: Make sure php is in your path, and edit this line so it has
   // the correct path to md5.php on your computer:
   var retObj = execCmdWithOutput("php \\BBS\\sbbs\\web\\myStuff\\md5.php \"" + pStr + "\"");
   if ((retObj.returnCode == 0) && (retObj.cmdOutput.length == 1))
      md5Str = retObj.cmdOutput[0];
   return md5Str;
}

// Converts a date in MM/DD/YY format to MM-DD-YYYY format.
//
// Paramters:
//  pDate: The date (in MM/DD/YY format)
//
// Return value: The date in MM-DD-YYYY format
function MMDDYYtoMMDDYYYY(pDate)
{
   if (typeof(pDate) != "string")
      return "0000-00-00";

   var theDate = "0000-00-00";
   if (/^[0-9][0-9]\/[0-9][0-9]\/[0-9][0-9]$/.test(pDate))
   {
      var monthStr = pDate.substr(0, 2);
      var dayStr = pDate.substr(3, 2);
      var yearStr02 = pDate.substr(6, 2);
      var yearStr = (+(yearStr02) < 40 ? "20" : "19") + yearStr02;
      theDate = monthStr + "-" + dayStr + "-" + yearStr;
   }
   return theDate;
}

// Converts a date in MM/DD/YY format to YYYY-MM-DD format.
//
// Paramters:
//  pDate: The date (in MM/DD/YY format)
//
// Return value: The date in YYYY-MM-DD format
function MMDDYYtoYYYYMMDD(pDate)
{
   if (typeof(pDate) != "string")
      return "0000-00-00";

   var theDate = "0000-00-00";
   if (/^[0-9][0-9]\/[0-9][0-9]\/[0-9][0-9]$/.test(pDate))
   {
      var monthStr = pDate.substr(0, 2);
      var dayStr = pDate.substr(3, 2);
      var yearStr02 = pDate.substr(6, 2);
      var yearStr = (+(yearStr02) < 40 ? "20" : "19") + yearStr02;
      theDate = yearStr + "-" + monthStr + "-" + dayStr;
   }
   return theDate;
}

// This function executes an OS command and returns its output as an
// array of strings.  The reason this function was written is that
// system.popen() is only functional in UNIX.
//
// Parameters:
//  pCommand: The command to execute
//
// Return value: An object containing the following properties:
//               returnCode: The return code of the OS command
//               cmdOutput: An array of strings containing the program's output.
function execCmdWithOutput(pCommand)
{
   var retObj = new Object();
   retObj.returnCode = 0;
   retObj.cmdOutput = new Array();

   if ((pCommand == undefined) || (pCommand == null) || (typeof(pCommand) != "string"))
      return retObj;

   // Execute the command and redirect the output to a file in the
   // node's directory.  system.exec() returns the return code that the
   // command returns; generally, 0 means success and non-zero means
   // failure (or an error of some sort).
   const tempFilename = system.temp_dir + "usersOutput1.temp";
   retObj.returnCode = system.exec(pCommand + " >" + tempFilename + " 2>&1");
   // Read the temporary file and populate retObj.cmdOutput with its
   // contents.
   var tempFile = new File(tempFilename);
   if (tempFile.open("r"))
   {
      if (tempFile.length > 0)
      {
         var fileLine = null;
         while (!tempFile.eof)
         {
            fileLine = tempFile.readln(2048);

            // fileLine should be a string, but I've seen some cases
            // where it isn't, so check its type.
            if (typeof(fileLine) != "string")
               continue;

            retObj.cmdOutput.push(fileLine);
         }
      }

      tempFile.close();
   }

   // Remove the temporary file, if it exists.
   if (file_exists(tempFilename))
      file_remove(tempFilename);

   return retObj;
}

// This function sends a local email to a user.
//
// Paramters:
//  pUserNum: The number of the user to send email to
//  pSubj: The subject of the email
//  pMessage: The email message
//  pFromUserNum: The number of the user to use as the message sender.
//                This is optional; if not specified, the current user
//                will be used.
//  pLocal: Boolean - Whether or not this is to be a local email.  If true,
//          the email will be sent to the user locally; if false, the message
//          will be sent to the user's Internet email address, if they have
//          one (will revert back to local if not).  This parameter is optional;
//          if not specified, the behavior will default to local.
//  pShowFinalStatus: Boolean - Whether or not to output the final status
//                    if the message was sent.  This is optional; this
//                    behavior defaults to false.
//
// Return value: String - Blank on success, or message on failure.
function emailUser(pUserNum, pSubj, pMessage, pFromUserNum, pLocal, pShowFinalStatus)
{
   // Return if the parameters are invalid.
   if ((typeof(pUserNum) != "number") || (typeof(pSubj) != "string") ||
       (typeof(pMessage) != "string"))
   {
      return ("Invalid parameters");
   }
   if (pUserNum == 0)
      return ("Invalid user number (0)");
   if (pMessage.length == 0)
      return ("Not sending an empty message");

   // Load the user specified by pUserNum.  If it's a deleted user, then
   // just return.
   var toUser = new User(pUserNum);
   if (toUser.settings & USER_DELETED)
      return ("The user is marked as deleted");

	var msgbase = new MsgBase("mail");
	if ((msgbase.open != undefined) && !msgbase.open())
	{
      msgbase.close();
		console.print("\r\nnhrERROR: y" + msgbase.last_error + "n\r\n");
		console.pause();
		return ("Error opening the message area: " + msgbase.last_error);
	}

   // Create the message header, and send the message.
   var header = new Object();
	header.to = toUser.alias;
	// Determine whether to send as local or Internet email.
	var local = true;
   if (typeof(pLocal) == "boolean")
      local = pLocal;
   // If sending via Internet and the user's netmail address is syntactially
   // valid, then send via Internet email.  Otherwise, send locally.
   if (!local && /^.+@.+\..{2,3}$/.test(toUser.netmail))
   {
      header.to_net_addr = toUser.netmail;
      header.to_net_type = NET_INTERNET;
   }
   else
   {
      // Sending locally.
      local = true;
      header.to_net_type = NET_NONE;
      header.to_ext = toUser.number;
   }

	// Determine what user number and net address to use for the sender (depending on the
	// value of pFromUserNum).
	var fromUserNum = 1;
	var fromNetAddr = system.username(1);
	if ((typeof(pFromUserNum) == "number") && (pFromUserNum > 0) && (pFromUserNum <= system.lastuser))
	{
      // If pFromUserNum doesn't specify a deleted user, then go ahead
      // and use it.
      var fromUser = new User(pFromUserNum);
      if ((fromUser.settings & USER_DELETED) == 0)
      {
         fromUserNum = pFromUserNum;
         fromNetAddr = fromUser.netmail;
      }
	}
   header.from = system.username(fromUserNum);
   header.from_ext = fromUserNum;
   header.from_net_addr = fromNetAddr;
   header.from_net_type = NET_NONE;
	header.subject = pSubj;
	var saveRetval = msgbase.save_msg(header, pMessage);
	msgbase.close();

	if (!saveRetval)
	{
		console.print("\r\nnhrERROR: y" + msgbase.last_error + "n\r\np");
		return ("Error saving the message: " + msgbase.last_error);
	}

   if (pShowFinalStatus)
      console.print("n\r\nhbMessage sent.n\r\n");

   return "";
}

// This function sends a local email to a user.
//
// Paramters:
//  pSubBoardCode: Synchronet's internal code for the sub-board to post in
//  pTo: The name of the person to send the message to
//  pSubj: The subject of the email
//  pMessage: The email message
//  pFromUserNum: The number of the user to use as the message sender.
//                This is optional; if not specified, the current user
//                will be used.
//  pShowFinalStatus: Boolean - Whether or not to output the final status
//                    if the message was sent.  This is optional; this
//                    behavior defaults to false.
//
// Return value: String - Blank on success, or message on failure.
function postMsgToSubBoard(pSubBoardCode, pTo, pSubj, pMessage, pFromUserNum, pShowFinalStatus)
{
   // Return if the parameters are invalid.
   if (typeof(pSubBoardCode) != "string")
      return ("Sub-board code is not a string");
   if (typeof(pTo) != "string")
      return ("To name is not a string");
   if (pTo.length == 0)
      return ("The 'to' user name is blank");
   if (typeof(pSubj) != "string")
      return ("Subject is not a string");
   if (pSubj.length == 0)
      return ("The subject is blank");
   if (typeof(pMessage) != "string")
      return ("Message is not a string");
   if (pMessage.length == 0)
      return ("Not sending an empty message");
   if (typeof(pFromUserNum) != "number")
      return ("From user number is not a number");
   if ((pFromUserNum <= 0) || (pFromUserNum > system.lastuser))
      return ("Invalid user number");

   // LOad the user record specified by pFromUserNum.  If it's a deleted user,
   // then return an error.
   var fromUser = new User(pFromUserNum);
   if (fromUser.settings & USER_DELETED)
      return ("The 'from' user is marked as deleted");

	var msgbase = new MsgBase(pSubBoardCode);
	if ((msgbase.open != undefined) && !msgbase.open())
	{
      msgbase.close();
		console.print("\r\nnhrERROR: y" + msgbase.last_error + "n\r\n");
		console.pause();
		return ("Error opening the message area: " + msgbase.last_error);
	}

   // Create the message header, and send the message.
   var header = new Object();
	header.to = pTo;
   header.from_net_type = NET_NONE;
   header.to_net_type = NET_NONE;
   header.from = fromUser.alias;
   header.from_ext = fromUser.number;
   header.from_net_addr = fromUser.netmail;
	header.subject = pSubj;
	var saveRetval = msgbase.save_msg(header, pMessage);
	msgbase.close();

	if (!saveRetval)
	{
		console.print("\r\nnhrERROR: y" + msgbase.last_error + "n\r\np");
		return ("Error saving the message: " + msgbase.last_error);
	}

   if (pShowFinalStatus)
      console.print("n\r\nhbMessage sent.n\r\n");

   return "";
}