/* This script contains importUserAcctInVBulletin(), which inserts a
 * Synchronet user account into vBulletin for my BBS.  This file also
 * includes functions that it uses.
 *
 * Other files required:
 * DDMisc.js (in the same directory)
 * MySQL.js (in the same directory)
 *
 * Date       Author          Description
 * 2010-01-03 Eric Oulashin   Created
 * 2010-01-04 Eric Oulashin   Continued working on this
 * 2010-01-05 Eric Oulashin   Continued working on this
 * 2010-01-06 Eric Oulashin   Updated the code for inserting & deleting vBulletin
 *                            users to also update the userfield and usertextfield
 *                            tables using the user's userid.
 * 2010-03-06 Eric Oulashin   Fixed a bug in importUserAcctInVBulletinByNumber()
 *                            where if it failed, it would try to use theUser
 *                            when theUser doesn't exist there.
 */

load("sbbsdefs.js");

// Determine the location of this script (its startup directory).
// The code for figuring this out is a trick that was created by Deuce,
// suggested by Rob Swindell.  I've shortened the code a little.
var gStartupPath = '.';
try { throw dig.dist(dist); } catch(e) { gStartupPath = e.fileName; }
gStartupPath = backslash(gStartupPath.replace(/[\/\\][^\/\\]*$/,''));

load(gStartupPath + "MySQL.js");

// Settings for various things
var VBULLETIN_DBNAME = "vbulletin"; // TODO: Update this so it's correct for your BBS
var VERIFIED_USER_LEVEL = 50; // TODO: Update this so it's correct for your BBS

// Global variables
// These 3 variables are used for sending vBulletin notification emails to users.
var gVBullEmailSubj = system.name + " BBS - vBulletin web interface for message areas";
// Message body format string for internet email
var gVBullIEmailBodyFormat = "Hi - You are receiving this message because you are a member of "
                  + system.name + " BBS (at " + system.inet_addr + ").  This is an automated "
                  + "email to inform you that you now have access to "+ system.name + "'s "
                  + "vBulletin web interface for its message areas.  This provides a rich set "
                  + "of features for reading and posting messages.  You can access "
                  + system.name + "'s vBulletin web interface using the following URL:\r\n"
                  + "http://" + system.inet_addr + "/forums\r\n\r\n"
                  + "Your username for vBulletin access is: %s\r\n"
                  + "Your password for vBulletin access is the same as the password you used "
                  + "when you created an account on " + system.name + ", but with all"
                  + "upper-case letters.\r\n\r\n"
                  + "If you have any problems accessing or logging into the vBulletin "
                  + "web interface, feel free to send me an email.\r\n\r\n"
                  + "%s"
// Message body format string for local email
var gVBullLEmailBodyFormat = "Hi - This is an automated email to inform you that you now have access to\r\n"
                            + system.name + "'s vBulletin web interface for its message areas.\r\n"
                            + "This provides a rich set of features for reading and posting messages.\r\n"
                            + "You can access the vBulletin web interface using the following URL:\r\n"
                            + "http://" + system.inet_addr + "/forums\r\n\r\n"
                            + "Your username for vBulletin access is: %s\r\n"
                            + "Your password for vBulletin access is the same as the password you used when\r\n"
                            + "you created an account on " + system.name + ", but with\r\n"
                            + "all upper-case letters.\r\n\r\n"
                            + "If you have any problems accessing or logging into the vBulletin web\r\n"
                            + "interface, feel free to send me an email.\r\n\r\n"
                            + "%s";


// Imports a Synchronet user account into vBulletin.  In case a vBulletin
// account already exists with the user's alias/name, it will be deleted
// before being added (this is done to avoid running MySQL more than once).
//
// Paramters:
//  pUser: A Synchronet User object containing information about the user
//         to import.
//
// Return value: An object that contains the following properties:
//               returnCode: The return code from the SQL insert command.
//                           0 is good, non-zero is bad.
//               cmdOutput: An array of output strings from the SQL
//                          insert command
//               vBulletinUsername: The user's username in vBulletin
function importUserAcctInVBulletin(pUser)
{
   // Don't import certain user records
   var userAliasUpper = pUser.alias.toUpperCase();
   if ((pUser.settings & USER_DELETED) ||
       (pUser.security.level < VERIFIED_USER_LEVEL) ||
       (userAliasUpper == "VBULLETIN") ||
       (userAliasUpper == "GUEST"))
   {
      var returnObj = new Object();
      returnObj.returnCode = -1;
      returnObj.cmdOutput = new Array();
      returnObj.cmdOutput.push("Not an importable user");
      returnObj.username = pUser.alias;
      return returnObj;
   }

   // Note: This script assumes the users' birthdates are in MM/DD/YY format,
   // because that's the date format I have set for my BBS.

   // I need these from the user:
   // vbulletin.user col      Synchronet variable/what to use
   // ------------------      -------------------------------
   // userid                  N/A - Auto-increment field
   // usergroupid             2 (registered user)
   // username                pUser.alias
   // email                   pUser.netmail
   // birthday                pUser.birthdate
   // salt                    Random 3 characters
   // password                pUser.security.password (encoded - See above comment)
   /*
   console.print("Username: " + pUser.alias + "\r\nLevel: " + pUser.security.level +
                 "\r\nEmail: " + pUser.netmail + "\r\nBirthday: " +
                 MMDDYYtoMMDDYYYY(pUser.birthdate) + "\r\nSalt: " + passwdObj.salt +
                 "\r\nPassword: " + passwdObj.password + "\r\n\r\n");
   */

   // Get the "salt" and vBulletin password for the user
   var passwdObj = getVBulletinPassword(pUser.security.password);

   // Get a username for the user that is safe for SQL and vBulletin.
   var vBulletinUsername = getVBUlletinUsername(pUser.alias);

   // Get the current time, for use with setting some dates in vBulletin's
   // user table
   var now = time();

   // Construct the SQL command to insert the user into vbulletin's user table.
   // This query will delete the current user account (in case the user exists
   //  already) and re-create the user account.
   var SQLQuery = getVBulletinUserDeleteSQL(vBulletinUsername)
                // Note: REPLACE INTO will insert and replace an old record, if it exists.
                + "INSERT INTO user (usergroupid, username, email, birthday, salt, "
                + "password, joindate, lastvisit, lastactivity, passworddate, posts) "
                + "VALUES (2, '" + vBulletinUsername + "', '" + pUser.netmail + "', '"
                + MMDDYYtoMMDDYYYY(pUser.birthdate) + "', '" + passwdObj.salt + "', '"
                + passwdObj.password + "', " + pUser.stats.firston_date + ", "
                + pUser.stats.laston_date + ", " + pUser.stats.laston_date + ", '"
                + strftime("%Y-%m-%d", now) + "', " + pUser.stats.total_posts + ");";
   // In addition to inserting into the user table, we also need to insert the user's
   // user ID into the userfield and usertextfield tables.
   SQLQuery += "INSERT INTO userfield (userid, temp, field1, field2, field3, field4) VALUES "
             + "((select userid from user where username='" + vBulletinUsername + "'), NULL, '', '', '', '');"
             + "INSERT INTO usertextfield (userid, subfolders, pmfolders, buddylist, ignorelist, "
             + "signature, searchprefs, rank) VALUES "
             + "((select userid from user where username='" + vBulletinUsername
             + "'), NULL, NULL, NULL, NULL, NULL, NULL, NULL);";
   //console.print("SQL query:\r\n" + SQLQuery + "\r\n\r\n"); // Temporary
   // Run the SQL command to insert the user into the MySQL database for vbulletin
   var returnObj = runMySQLQuery(SQLQuery);

   returnObj.vBulletinUsername = vBulletinUsername;

   return returnObj;
}

// Imports a Synchronet user account into vBulletin by user #.
//
// Paramters:
//  pUserNum: The user record number of the user to import
//                   
//
// Return value: An object that contains the following properties:
//               returnCode: The return code from the SQL insert command.
//                           0 is good, non-zero is bad.
//               cmdOutput: An array of output strings from the SQL
//                          insert command
//               username: The user's username in Synchronet
//               vBulletinUsername: The user's username in vBulletin
function importUserAcctInVBulletinByNumber(pUserNum)
{
   var returnObj = null;
   // Make sure pUserNum is valid.
   if ((pUserNum > 0) && (pUserNum <= system.lastuser))
   {
      var theUser = new User(pUserNum);
      returnObj = importUserAcctInVBulletin(theUser);
      returnObj.username = theUser.alias;
   }
   else
   {
      // pUserNum is not valid.
      returnObj = new Object();
      returnObj.returnCode = -1;
      returnObj.cmdOutput = new Array();
      returnObj.cmdOutput.push("User number is out of range");
      returnObj.username = "";
      returnObj.vBulletinUsername = "";
   }

   return returnObj;
}

// Deletes a vBulletin account.
// Paramters:
//  pUser: A Synchronet User object containing information about the user
//         to delete.
//  pVBUlletinUsername: The user's vBulletin username (if already retrieved).
//                      This parameter is optional.  If not specified, then
//                      this function will figure out the user's vBulletin
//                      username.
//
// Return value: An object that contains the following properties:
//               returnCode: The return code from the SQL insert command.
//                           0 is good, non-zero is bad.
//               cmdOutput: An array of output strings from the SQL
//                          insert command
//               vBulletinUsername: The user's vBulletin username
function deleteVBUlletinAcct(pUser, pVBUlletinUsername)
{
   var vBulletinUsername = null;
   if ((typeof(pVBUlletinUsername) == "string") && (pVBUlletinUsername.length > 0))
      vBulletinUsername = pVBUlletinUsername;
   else
      vBulletinUsername = getVBUlletinUsername(pUser.alias);
   var returnObj = runMySQLQuery(getVBulletinUserDeleteSQL(vBulletinUsername));
   returnObj.vBulletinUsername = vBulletinUsername;
   return returnObj;
}

// Returns SQL for deleting a vBulletin user account.
//
// Parameters:
//  pUsername: The vBulletin username to delete (already converted to a
//             username safe for SQL and vBulletin using getVBUlletinUsername())
//
// Return value: The SQL for deleting the user from the vBulletin database
function getVBulletinUserDeleteSQL(pUsername)
{
   return ("use " + VBULLETIN_DBNAME + ";"
            // We need to delete the user's userid from the userfield and usertextfield
            // tables, then delete the user from the user table.
            + "DELETE FROM userfield WHERE userid=(SELECT userid FROM user WHERE username='"
            + pUsername + "');"
            + "DELETE FROM usertextfield WHERE userid=(SELECT userid FROM user WHERE username='"
            + pUsername + "');"
            + "DELETE FROM user WHERE username='" + pUsername + "';");
}

// Deletes a user's vBulletin account, by user number.
//
// Parameters:
//  pUserNum: The user's Synchronet user number
//
// Return value: An object that contains the following properties:
//               returnCode: The return code from the SQL insert command.
//                           0 is good, non-zero is bad.
//               cmdOutput: An array of output strings from the SQL
//                          insert command
//               username: The user's Synchronet username
//               vBulletinUsername: The user's vBulletin user name
function deleteVBUlletinAcctByNumber(pUserNum)
{
   var returnObj = null;
   // Make sure pUserNum is valid.
   if ((pUserNum > 0) && (pUserNum <= system.lastuser))
   {
      var theUser = new User(pUserNum);
      var vBulletinUsername = getVBUlletinUsername(theUser.alias);
      returnObj = deleteVBUlletinAcct(theUser, vBulletinUsername);
      returnObj.username = theUser.alias;
   }
   else
   {
      // pUserNum is not valid.
      returnObj = new Object();
      returnObj.returnCode = -1;
      returnObj.cmdOutput = new Array();
      returnObj.cmdOutput.push("Invalid user number");
      returnObj.username = "";
      returnObj.vBulletinUsername = "";
   }
   return returnObj;
}

// Returns a vBullein-encrypted version of a password.
//
// Parameters:
//  pPassword: The password to encrypt
//
// Return value: An object containing the following properties:
//               salt: The random 3-character string used in the password
//               password: The password (string)
function getVBulletinPassword(pPassword)
{
   // vBulletin stores user account passwords in the following format:
   // $password_hash = md5(md5($password_text) . $user_salt);
   // $user_salt is a random three character string stored in the user table as 'salt'.
   // That information came from the following URL:
   // http://www.vbulletin.com/forum/showthread.php?211445-vbulletin-password-encryption

   // This script assembles the vBulletin password using my md5.php script.
   var retObj = new Object();
   retObj.salt = randomChar() + randomChar() + randomChar();
   retObj.password = md5(md5(pPassword) + retObj.salt);
   return retObj;
}

// Converts a username to a username that is safe for vBulletin (strips
// characters that are not SQL-safe/vBulletin-safe).
//
// Parameters:
//  pUsername: The username to convert
//
// Return value: A version of the username that is safe for SQL & vBulletin
function getVBUlletinUsername(pUsername)
{
   return (pUsername.replace("'", ""));
}