/* This is a script for importing a Synchronet user account into vBulletin.
 * This script is meant to be run from the command line using Synchronet's
 * jsexec utility.
 *
 * Author: Eric Oulashin (AKA Nightfox)
 * BBS: Digital Distortion
 * BBS address: digdist.bbsindex.com
 *
 * Date       User              Description
 * 2010-03-06 Eric Oulashin     Created
 */

// TODO: Make sure the path for this load statement is correct for
// your BBS.
load("vBulletin.js");

// If the argument count is not 1, output a usage message and exit.
if (argc != 1)
{
   print("Usages:\r\n");
   print("Import a single user to vBulletin: jsexec SyncUserToVBulletin.js <user #>\r\n");
   print("Import a range of users to vBulletin: jsexec SyncUserToVBulletin.js <user #>-<user #>\r\n");
   exit(1);
}

// Set up the start & end user number variables.
var startUser = 0;
var endUser = null;
if (/^[0-9]+$/.test(argv[0]))
   startUser = +(argv[0]);
else if (/^[0-9]+-[0-9]+$/.test(argv[0]))
{
   var dashPos = argv[0].indexOf("-");
   startUser = +(argv[0].substr(0, dashPos));
   endUser = +(argv[0].substr(dashPos+1));
}

// Range-check the user numbers
if ((startUser < 2) || (startUser > system.lastuser))
{
   print("Starting user number is out of range.  It must be between 2 and " +
          system.lastuser + ".\r\n");
   exit(2);
}
if (endUser != null)
{
   if ((endUser < 2) || (endUser > system.lastuser))
   {
      print("Ending user number is out of range.  It must be between 2 and " +
             system.lastuser + ".\r\n");
      exit(3);
   }
}

// Output the user number range
print("Start user #: " + startUser);
if (endUser != null)
   print("End user #: " + endUser);
print("\r\n");

// If only startUser is specified, then import only that user.  Otherwise,
// iterate through the range of startUser to endUser, inserting the
// users into vBulletin.
if (endUser == null)
   importUserIntoVBulletinByNum(startUser);
else
{
   var theUser = new User(1);
   for (var userNum = startUser; userNum <= endUser; ++userNum)
   {
      theUser.number = userNum;
      // If the user is not deleted or inactive, then import
      // the user into vBulletin.
      if (!(theUser.settings & (USER_DELETED|USER_INACTIVE)))
         importUserIntoVBulletinByNum(userNum);
   }
}

exit(0);

// End of script execution.

/////////////////////////////////////////////////////////////
// Functions

// Imports a Synchronet user account into vBulletin by user number.
//
// Parameters:
//  pUserNum: The Synchronet user number
function importUserIntoVBulletinByNum(pUserNum)
{
   var retObj = importUserAcctInVBulletinByNumber(pUserNum);
   if (retObj.returnCode == 0)
   {
      print("Successfully imported " + retObj.username + " into vBulletin.\r\n");
      print("Their vBulletin username is " + retObj.vBulletinUsername + "\r\n");
   }
   else
   {
      print("Failed to import the user into vBulletin!\r\n");
   }
}